﻿using System.Linq;
using System.Windows;
using System.Windows.Documents;

namespace Sketchables
{
  /// <summary>
  /// Parses text that contains formatting tags into formatted segments.
  /// </summary>
  public class TextParser : TextParserBase
  {
    /// <summary>
    /// Inits the parser with the supported tags.
    /// </summary>
    public TextParser() : base('*', '_', ':')
    {
    }



    /// <summary>
    /// Applies formatting to the submitted run. As runs cannot be
    /// nested, we apply all formatting that is derived to the run,
    /// starting from the top and going down to the current element,
    /// which allows us to override formats, should we have to (currently
    /// not an issue, but might become one with more sophisticated tags). 
    /// </summary>
    protected override void FormatTextBlock(Run run, TextSegment segment)
    {
      var segments = segment.GetParentGraph().Reverse();
      foreach (var s in segments)
      {
        switch (s.SegmentTag)
        {
          case '*':
            run.FontWeight = FontWeights.Bold;
            break;
          case '_':
            run.FontStyle = FontStyles.Italic;
            break;
          case ':':
            run.TextDecorations = TextDecorations.Underline;
            break;
        }
      }
    }


    /// <summary>
    /// Unescapes a given text.
    /// </summary>
    protected override string Unescape(string text)
    {
      foreach (char tagChar in Tags)
      {
        string tag = tagChar.ToString();
        text = text.Replace("\\" + tag, tag);
      }

      return text;
    }
  }
}